<?php

namespace Drupal\spammaster;

use Drupal\Core\Database\Connection;
use Drupal\Core\Messenger\MessengerInterface;
use Symfony\Component\HttpFoundation\RequestStack;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Session\AccountProxy;
use GuzzleHttp\Client;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\State\StateInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\StringTranslation\TranslationInterface;
use Drupal\Component\Serialization\Json;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Class service.
 */
class SpamMasterElusiveService {
  use StringTranslationTrait;

  /**
   * The database connection object.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $connection;

  /**
   * The Messenger service.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * The session account interface.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $account;

  /**
   * Symfony\Component\HttpFoundation\RequestStack definition.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected $requestStack;

  /**
   * Client.
   *
   * @var \GuzzleHttp\Client
   */
  protected $httpClient;

  /**
   * The Config Factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The state.
   *
   * @var \Drupal\Core\State\StateInterface
   */
  protected $state;

  /**
   * The String Translation..
   *
   * @var \Drupal\Core\StringTranslation\TranslationInterface
   */
  protected $stringTranslation;

  /**
   * Drupal\Core\Session\AccountProxy definition.
   *
   * @var \Drupal\Core\Session\AccountProxy
   */
  protected $currentUser;

  /**
   * The SpamMasterElusiveService service.
   *
   * @var \Drupal\spammmaster\SpamMasterElusiveService
   */
  protected $spamelusive;

  /**
   * The SpamMasterCollectService Service.
   *
   * @var \Drupal\spammaster\SpamMasterCollectService
   */
  protected $collectService;

  /**
   * The SpamMasterUserService Service.
   *
   * @var \Drupal\spammaster\SpamMasterUserService
   */
  protected $userService;

  /**
   * The form class.
   *
   * @var \Drupal\Core\Form\FormInterface
   */
  protected $form;

  /**
   * The form class.
   *
   * @var \Drupal\Core\Form\FormInterface
   */
  protected $formstate;

  /**
   * The form class.
   *
   * @var \Drupal\Core\Form\FormInterface
   */
  protected $spamFormId;

  /**
   * The SpamMasterCleanUpService Service.
   *
   * @var \Drupal\spammaster\SpamMasterCleanUpService
   */
  protected $cleanUpService;

  /**
   * {@inheritdoc}
   */
  public function __construct(Connection $connection, MessengerInterface $messenger, AccountInterface $account, RequestStack $requestStack, Client $httpClient, ConfigFactoryInterface $configFactory, StateInterface $state, TranslationInterface $stringTranslation, AccountProxy $current_user, SpamMasterCollectService $collectService, SpamMasterUserService $userService, SpamMasterCleanUpService $cleanUpService) {
    $this->connection = $connection;
    $this->messenger = $messenger;
    $this->account = $account;
    $this->requestStack = $requestStack;
    $this->httpClient = $httpClient;
    $this->configFactory = $configFactory;
    $this->state = $state;
    $this->stringTranslation = $stringTranslation;
    $this->currentUser = $current_user;
    $this->collectService = $collectService;
    $this->userService = $userService;
    $this->cleanUpService = $cleanUpService;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('database'),
      $container->get('messenger'),
      $container->get('current_user'),
      $container->get('request_stack'),
      $container->get('http_client'),
      $container->get('config.factory'),
      $container->get('state'),
      $container->get('string_translation'),
      $container->get('current_user'),
      $container->get('spammaster.collect_service'),
      $container->get('spammaster.user_service'),
      $container->get('spammaster.clean_service')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function spamMasterElusiveCheck($spamelusive) {

    $this->spamelusive = $spamelusive;
    $spammaster_date = date("Y-m-d H:i:s");
    $spammaster_settings = $this->configFactory->getEditable('spammaster.settings');
    $spammaster_license = $spammaster_settings->get('spammaster.license_key');
    $spammaster_status = $this->state->get('spammaster.license_status');
    $spammaster_total_block_count = $this->state->get('spammaster.total_block_count');
    $spammasterWhiteTransient = $this->state->get('spammaster.white_transient');
    if ('VALID' === $spammaster_status || 'MALFUNCTION_1' === $spammaster_status || 'MALFUNCTION_2' === $spammaster_status) {

      // Call collect service.
      $collectnow = $this->requestStack->getCurrentRequest()->getClientIp();
      $spammaster_collect_service = $this->collectService;
      $is_collected = $spammaster_collect_service->spamMasterGetCollect($collectnow);
      $spamcollection = Json::decode($is_collected);
      $spammasterip = $spamcollection['spammasterip'];
      $spammasteragent = $spamcollection['spammasteragent'];
      $spammasterreferer = $spamcollection['spammasterreferer'];
      $spammasterurl = $spamcollection['spammasterurl'];

      // User service call.
      $spamuser = $this->currentUser->id();
      $spammaster_user_service = $this->userService;
      $is_user = $spammaster_user_service->spamMasterGetUser($spamuser);
      $spamuserdata = Json::decode($is_user);
      $spammasteruserid = $spamuserdata['ID'];
      $spammasterusername = $spamuserdata['username'];

      // Exempt whitelist.
      $spammaster_white_query = $this->connection->query("SELECT white FROM {spammaster_white} WHERE white = :ip", [
        ':ip' => $spammasterip,
      ])->fetchField();
      // Check white transient if on.
      if ('1' === $spammasterWhiteTransient) {
        // Call clean-up service to delete old transients.
        $spammaster_cleanup_service = $this->cleanUpService;
        $spammaster_cleanup_service->spamMasterCleanUpTransNow($spammasterWhiteTransient);
        // Exempt transient whitelist.
        $spammaster_white_trans_query = $this->connection->query("SELECT spamvalue FROM {spammaster_keys} WHERE spamkey = :key AND spamvalue = :value", [
          ':key' => 'white-transient-haf',
          ':value' => $spammasterip,
        ])->fetchField();
      }
      else {
        $spammaster_white_trans_query = FALSE;
      }
      // White positive, or administrator log insert.
      $user_roles = $this->account->getRoles();
      if (!empty($spammaster_white_query) || !empty($spammaster_white_trans_query) || in_array('administrator', $user_roles)) {
        return 'BAIL';
      }
      else {
        // Buffer db check.
        $spammaster_spam_buffer_query = $this->connection->query("SELECT threat FROM {spammaster_threats} WHERE threat = :ip", [
          ':ip' => $spammasterip,
        ])->fetchField();
        // Buffer db positive, throw error, log insert.
        if (!empty($spammaster_spam_buffer_query)) {
          $spammaster_total_block_count_1 = ++$spammaster_total_block_count;
          $this->state->set('spammaster.total_block_count', $spammaster_total_block_count_1);
          $this->connection->insert('spammaster_keys')->fields([
            'date' => $spammaster_date,
            'spamkey' => 'spammaster-firewall',
            'spamvalue' => 'Spam Master: firewall BUFFER BLOCK, Ip: ' . $spammasterip . ', Agent: ' . $spammasteragent . ', User ID: ' . $spammasteruserid . ', Username: ' . $spammasterusername,
          ])->execute();
          return 'ELUSIVE';
        }
        else {

          // Exclude malformed or internal links.
          if (!empty($spammasterurl)) {
            if (stripos($spammasterurl, 'autodiscover.xml') !== FALSE) {
              return 'BAIL';
            }
          }

          // phpcs:ignore DrupalPractice.Variables.GetRequestData.SuperglobalAccessed
          $spampost = $_POST;
          if (!empty($spampost) && is_array($spampost)) {
            $spampoststr = str_replace('=', ' ', urldecode(http_build_query($spampost, '', ' ')));
            // Exclusions by key.
            $spammaster_exempt_key = $this->connection->query("SELECT spamvalue FROM {spammaster_keys} WHERE spamkey = :key AND POSITION(spamvalue IN :exclude) > :value", [
              ':key' => 'exempt-key',
              ':exclude' => $spampoststr,
              ':value' => 0,
            ])->fetchField();
            if (!empty($spammaster_exempt_key)) {
              return 'BAIL';
            }
            // Exclusions by value.
            $spammaster_exempt_value = $this->connection->query("SELECT spamvalue FROM {spammaster_keys} WHERE spamkey = :key AND POSITION(spamvalue IN :exclude) > :value", [
              ':key' => 'exempt-value',
              ':exclude' => $spampoststr,
              ':value' => 0,
            ])->fetchField();
            if (!empty($spammaster_exempt_value)) {
              return 'BAIL';
            }
            // Web api check. Create data to be posted.
            $blog_threat_type = 'HAF';
            $result_post_content_json  = Json::encode($spampost);
            $result_post_content_trim  = substr($result_post_content_json, 0, 963);
            $result_post_content_clean = strip_tags($result_post_content_trim);
            if (empty($result_post_content_clean)) {
              $result_post_content_clean = 'is_elusive_d';
            }
            // Collect email to scan.
            preg_match('/[_a-z0-9-]+(\.[_a-z0-9-]+)*@[a-z0-9-]+(\.[a-z0-9-]+)*(\.[a-z]{2,3})/i', $spampoststr, $matches);
            if ($matches) {
              foreach ($matches as $key => $val) {
                $resultKey = $key;
                if ($resultKey) {
                }
                if (filter_var($val, FILTER_VALIDATE_EMAIL)) {
                  $spammasteremail = substr($val, 0, 256);
                  break;
                }
                else {
                  $spammasteremail = 'haf@' . rand(10000000, 99999999) . '.drup';
                }
              }
            }
            else {
              $spammasteremail = 'haf@' . rand(10000000, 99999999) . '.drup';
            }
            // Proceed with post.
            $blog_web_address = $this->requestStack->getCurrentRequest()->getHost();
            $address_unclean = $blog_web_address;
            $address = preg_replace('#^https?://#', '', $address_unclean);
            @$blog_server_ip = $_SERVER['SERVER_ADDR'];
            // If empty ip.
            if (empty($blog_server_ip) || $blog_server_ip == '0') {
              @$blog_server_ip = 'I ' . gethostbyname($_SERVER['SERVER_NAME']);
            }
            $spam_master_leaning_url = 'https://www.spammaster.org/core/learn/get_learn_haf.php';

            // Call drupal hhtpclient.
            $client = $this->httpClient;
            // Post data.
            $request = $client->post($spam_master_leaning_url, [
              'form_params' => [
                'blog_license_key'    => $spammaster_license,
                'blog_threat_ip'      => $spammasterip,
                'blog_threat_user'    => $is_user,
                'blog_threat_type'    => $blog_threat_type,
                'blog_threat_email'   => $spammasteremail,
                'blog_threat_content' => 'is_elusive_d - ' . $result_post_content_clean,
                'blog_threat_agent'   => $spammasteragent,
                'blog_threat_refe'    => $spammasterreferer,
                'blog_threat_dest'    => $spammasterurl,
                'blog_web_adress'     => $address,
                'blog_server_ip'      => $blog_server_ip,
              ],
            ]);
            // Decode json data or $request->getBody()->getContents().
            $response = Json::decode($request->getBody());
            if (empty($response['threat'])) {
              // Log clean rbl check.
              $this->connection->insert('spammaster_keys')->fields([
                'date' => $spammaster_date,
                'spamkey' => 'spammaster-firewall',
                'spamvalue' => 'Spam Master: firewall RBL CHECK, Ip: ' . $spammasterip . ', Agent: ' . $spammasteragent . ', User ID: ' . $spammasteruserid . ', Username: ' . $spammasterusername,
              ])->execute();
              // Set white transient if on.
              if ('1' === $spammasterWhiteTransient) {
                $this->connection->insert('spammaster_keys')->fields([
                  'date' => $spammaster_date,
                  'spamkey' => 'white-transient-haf',
                  'spamvalue' => $spammasterip,
                ])->execute();
              }
              return 'BAIL';
            }
            else {
              // Insert ip into buffer db.
              $this->connection->insert('spammaster_threats')->fields([
                'date' => $spammaster_date,
                'threat' => $spammasterip,
              ])->execute();
              // Web positive, return.
              $spammaster_total_block_count_1 = ++$spammaster_total_block_count;
              $this->state->set('spammaster.total_block_count', $spammaster_total_block_count_1);
              $this->connection->insert('spammaster_keys')->fields([
                'date' => $spammaster_date,
                'spamkey' => 'spammaster-firewall',
                'spamvalue' => 'Spam Master: firewall RBL BLOCK, Ip: ' . $spammasterip . ', Agent: ' . $spammasteragent . ', User ID: ' . $spammasteruserid . ', Username: ' . $spammasterusername,
              ])->execute();
              return 'ELUSIVE';
            }
          }
          else {
            return 'BAIL';
          }
        }
      }
    }
    else {
      return 'BAIL';
    }
  }

  /**
   * {@inheritdoc}
   */
  public function spamMasterIsNeedle($spamFormId) {

    // Force needle straw.
    $spammaster_exempt_needle_straw = $this->connection->query("SELECT spamvalue FROM {spammaster_keys} WHERE spamkey = :key AND POSITION(spamvalue IN :straw) > :value", [
      ':key' => 'exempt-needle-straw',
      ':straw' => $spamFormId,
      ':value' => 0,
    ])->fetchField();
    if (!empty($spammaster_exempt_needle_straw)) {
      return "STRAW";
    }
    else {
      // Hide sig but force needle.
      $spammaster_exempt_needle_hide_sig = $this->connection->query("SELECT spamvalue FROM {spammaster_keys} WHERE spamkey = :key AND POSITION(spamvalue IN :sighide) > :value", [
        ':key' => 'exempt-needle-sig-hide',
        ':sighide' => $spamFormId,
        ':value' => 0,
      ])->fetchField();
      if (!empty($spammaster_exempt_needle_hide_sig)) {
        return "SIGHIDDEN";
      }
      else {
        // Show sig but exempt needle.
        $spammaster_exempt_needle_show_sig = $this->connection->query("SELECT spamvalue FROM {spammaster_keys} WHERE spamkey = :key AND POSITION(spamvalue IN :sigshow) > :value", [
          ':key' => 'exempt-needle-sig-show',
          ':sigshow' => $spamFormId,
          ':value' => 0,
        ])->fetchField();
        if (!empty($spammaster_exempt_needle_show_sig)) {
          return "SIGSHOW";
        }
        else {
          // Exempt needle.
          $spammaster_exempt_needle = $this->connection->query("SELECT spamvalue FROM {spammaster_keys} WHERE spamkey = :key AND POSITION(spamvalue IN :exclude) > :value", [
            ':key' => 'exempt-needle',
            ':exclude' => $spamFormId,
            ':value' => 0,
          ])->fetchField();
          if (!empty($spammaster_exempt_needle)) {
            return "NEEDLE";
          }
          else {
            return "STRAW";
          }
        }
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function spamMasterFormData($form, $formstate, $spamFormId) {

    // Try to get the name or email.
    if (!empty($formstate->getValue('name')) && NULL !== $formstate->getValue('name') && !is_array($formstate->getValue('name'))) {
      if (filter_var($formstate->getValue('name'), FILTER_VALIDATE_EMAIL)) {
        $spammasteremail = $formstate->getValue('name');
      }
    }
    if (!empty($formstate->getValue('mail')) && NULL !== $formstate->getValue('mail')) {
      if (filter_var($formstate->getValue('mail'), FILTER_VALIDATE_EMAIL)) {
        $spammasteremail = $formstate->getValue('mail');
      }
    }
    if (!empty($formstate->getValue('field_email')) && NULL !== $formstate->getValue('field_email')) {
      if (is_array($formstate->getValue('field_email'))) {
        if (filter_var($formstate->getValue('field_email')[0]['value'], FILTER_VALIDATE_EMAIL)) {
          $spammastermessage = $formstate->getValue('field_email')[0]['value'];
        }
      }
      else {
        if (filter_var($formstate->getValue('field_email'), FILTER_VALIDATE_EMAIL)) {
          $spammasteremail = $formstate->getValue('field_email');
        }
      }
    }
    if (!empty($formstate->getValue('email')) && NULL !== $formstate->getValue('email')) {
      if (is_array($formstate->getValue('email'))) {
        if (filter_var($formstate->getValue('email')['mail_1'], FILTER_VALIDATE_EMAIL)) {
          $spammastermessage = $formstate->getValue('email')['mail_1'];
        }
      }
      else {
        if (filter_var($formstate->getValue('email'), FILTER_VALIDATE_EMAIL)) {
          $spammasteremail = $formstate->getValue('email');
        }
      }
    }
    if (!empty($formstate->getValue('e_mail')) && NULL !== $formstate->getValue('e_mail')) {
      if (filter_var($formstate->getValue('e_mail'), FILTER_VALIDATE_EMAIL)) {
        $spammasteremail = $formstate->getValue('e_mail');
      }
    }
    if (!empty($formstate->getValue('your_email')) && NULL !== $formstate->getValue('your_email')) {
      if (filter_var($formstate->getValue('your_email'), FILTER_VALIDATE_EMAIL)) {
        $spammasteremail = $formstate->getValue('your_email');
      }
    }
    if (!empty($formstate->getValue('ihre_e_mail_adresse')) && NULL !== $formstate->getValue('ihre_e_mail_adresse')) {
      if (filter_var($formstate->getValue('ihre_e_mail_adresse'), FILTER_VALIDATE_EMAIL)) {
        $spammasteremail = $formstate->getValue('ihre_e_mail_adresse');
      }
    }
    if (!empty($formstate->getValue('courriel')) && NULL !== $formstate->getValue('courriel')) {
      if (filter_var($formstate->getValue('courriel'), FILTER_VALIDATE_EMAIL)) {
        $spammasteremail = $formstate->getValue('courriel');
      }
    }
    if (!empty($formstate->getValue('email_address')) && NULL !== $formstate->getValue('email_address')) {
      if (filter_var($formstate->getValue('email_address'), FILTER_VALIDATE_EMAIL)) {
        $spammasteremail = $formstate->getValue('email_address');
      }
    }
    if (!empty($formstate->getValue('e_mailadres')) && NULL !== $formstate->getValue('e_mailadres')) {
      if (filter_var($formstate->getValue('e_mailadres'), FILTER_VALIDATE_EMAIL)) {
        $spammasteremail = $formstate->getValue('e_mailadres');
      }
    }
    if (!empty($formstate->getValue('mt_email')) && NULL !== $formstate->getValue('mt_email')) {
      if (filter_var($formstate->getValue('mt_email'), FILTER_VALIDATE_EMAIL)) {
        $spammasteremail = $formstate->getValue('mt_email');
      }
    }
    if (!empty($formstate->getValue('uw_gegevens')) && NULL !== $formstate->getValue('uw_gegevens')) {
      if (is_array($formstate->getValue('uw_gegevens'))) {
        if (filter_var($formstate->getValue('uw_gegevens')['email'], FILTER_VALIDATE_EMAIL)) {
          $spammastermessage = $formstate->getValue('uw_gegevens')['email'];
        }
      }
      else {
        if (filter_var($formstate->getValue('uw_gegevens'), FILTER_VALIDATE_EMAIL)) {
          $spammasteremail = $formstate->getValue('uw_gegevens');
        }
      }
    }
    if (!empty($formstate->getValue('your_email_address')) && NULL !== $formstate->getValue('your_email_address')) {
      if (filter_var($formstate->getValue('your_email_address'), FILTER_VALIDATE_EMAIL)) {
        $spammasteremail = $formstate->getValue('your_email_address');
      }
    }
    if (!empty($formstate->getValue('e_mail_adresse')) && NULL !== $formstate->getValue('e_mail_adresse')) {
      if (filter_var($formstate->getValue('e_mail_adresse'), FILTER_VALIDATE_EMAIL)) {
        $spammasteremail = $formstate->getValue('e_mail_adresse');
      }
    }
    if (!empty($formstate->getValue('kform_hpvbw_email')) && NULL !== $formstate->getValue('kform_hpvbw_email')) {
      if (is_array($formstate->getValue('kform_hpvbw_email'))) {
        if (filter_var($formstate->getValue('kform_hpvbw_email')['mail_1'], FILTER_VALIDATE_EMAIL)) {
          $spammastermessage = $formstate->getValue('kform_hpvbw_email')['mail_1'];
        }
      }
      else {
        if (filter_var($formstate->getValue('kform_hpvbw_email'), FILTER_VALIDATE_EMAIL)) {
          $spammasteremail = $formstate->getValue('kform_hpvbw_email');
        }
      }
    }
    if (!empty($formstate->getValue('email_de_la_persona_de_contacto')) && NULL !== $formstate->getValue('email_de_la_persona_de_contacto')) {
      if (filter_var($formstate->getValue('email_de_la_persona_de_contacto'), FILTER_VALIDATE_EMAIL)) {
        $spammasteremail = $formstate->getValue('email_de_la_persona_de_contacto');
      }
    }
    if (!empty($formstate->getValue('contact_email')) && NULL !== $formstate->getValue('contact_email')) {
      if (filter_var($formstate->getValue('contact_email'), FILTER_VALIDATE_EMAIL)) {
        $spammasteremail = $formstate->getValue('contact_email');
      }
    }
    if (!empty($formstate->getValue('anmeldung_e_mail')) && NULL !== $formstate->getValue('anmeldung_e_mail')) {
      if (is_array($formstate->getValue('anmeldung_e_mail'))) {
        if (filter_var($formstate->getValue('anmeldung_e_mail')['mail_1'], FILTER_VALIDATE_EMAIL)) {
          $spammastermessage = $formstate->getValue('anmeldung_e_mail')['mail_1'];
        }
      }
      else {
        if (filter_var($formstate->getValue('anmeldung_e_mail'), FILTER_VALIDATE_EMAIL)) {
          $spammasteremail = $formstate->getValue('anmeldung_e_mail');
        }
      }
    }
    if (!empty($formstate->getValue('activity_subscriber_email')) && NULL !== $formstate->getValue('activity_subscriber_email')) {
      if (filter_var($formstate->getValue('activity_subscriber_email'), FILTER_VALIDATE_EMAIL)) {
        $spammasteremail = $formstate->getValue('activity_subscriber_email');
      }
    }
    if (!empty($formstate->getValue('field_e_mail_address')) && NULL !== $formstate->getValue('field_e_mail_address')) {
      if (is_array($formstate->getValue('field_e_mail_address'))) {
        if (filter_var($formstate->getValue('field_e_mail_address')[0]['value'], FILTER_VALIDATE_EMAIL)) {
          $spammastermessage = $formstate->getValue('field_e_mail_address')[0]['value'];
        }
      }
      else {
        if (filter_var($formstate->getValue('field_e_mail_address'), FILTER_VALIDATE_EMAIL)) {
          $spammasteremail = $formstate->getValue('field_e_mail_address');
        }
      }
    }
    if (!empty($formstate->getValue('field_email_address')) && NULL !== $formstate->getValue('field_email_address')) {
      if (is_array($formstate->getValue('field_email_address'))) {
        if (filter_var($formstate->getValue('field_email_address')[0]['value'], FILTER_VALIDATE_EMAIL)) {
          $spammastermessage = $formstate->getValue('field_email_address')[0]['value'];
        }
      }
      else {
        if (filter_var($formstate->getValue('field_email_address'), FILTER_VALIDATE_EMAIL)) {
          $spammasteremail = $formstate->getValue('field_email_address');
        }
      }
    }
    if (!empty($formstate->getValue('field_e_mail_adresse')) && NULL !== $formstate->getValue('field_e_mail_adresse')) {
      if (is_array($formstate->getValue('field_e_mail_adresse'))) {
        if (filter_var($formstate->getValue('field_e_mail_adresse')[0]['value'], FILTER_VALIDATE_EMAIL)) {
          $spammastermessage = $formstate->getValue('field_e_mail_adresse')[0]['value'];
        }
      }
      else {
        if (filter_var($formstate->getValue('field_e_mail_adresse'), FILTER_VALIDATE_EMAIL)) {
          $spammasteremail = $formstate->getValue('field_e_mail_adresse');
        }
      }
    }
    if (!empty($formstate->getValue('correu_electronic')) && NULL !== $formstate->getValue('correu_electronic')) {
      if (filter_var($formstate->getValue('correu_electronic'), FILTER_VALIDATE_EMAIL)) {
        $spammasteremail = $formstate->getValue('correu_electronic');
      }
    }
    if (!empty($formstate->getValue('newsletter')) && NULL !== $formstate->getValue('newsletter')) {
      if (filter_var($formstate->getValue('newsletter'), FILTER_VALIDATE_EMAIL)) {
        $spammasteremail = $formstate->getValue('newsletter');
      }
    }
    if (empty($spammasteremail) || is_array($spammasteremail)) {
      $spammasteremail = 'drup@' . date('Ymdhis') . '.drup';
    }
    // Try to get the message.
    if (!empty($formstate->getValue('message')) && NULL !== $formstate->getValue('message')) {
      if (is_array($formstate->getValue('message'))) {
        $spammastermessage = $formstate->getValue('message')[0]['value'];
      }
      else {
        $spammastermessage = $formstate->getValue('message');
      }
    }
    if (!empty($formstate->getValue('bericht')) && NULL !== $formstate->getValue('bericht')) {
      if (is_array($formstate->getValue('bericht'))) {
        $spammastermessage = $formstate->getValue('bericht')[0]['value'];
      }
      else {
        $spammastermessage = $formstate->getValue('bericht');
      }
    }
    if (!empty($formstate->getValue('uw_bericht')) && NULL !== $formstate->getValue('uw_bericht')) {
      if (is_array($formstate->getValue('uw_bericht'))) {
        $spammastermessage = $formstate->getValue('uw_bericht')[0]['value'];
      }
      else {
        $spammastermessage = $formstate->getValue('uw_bericht');
      }
    }
    if (!empty($formstate->getValue('nachricht')) && NULL !== $formstate->getValue('nachricht')) {
      if (is_array($formstate->getValue('nachricht'))) {
        $spammastermessage = $formstate->getValue('nachricht')[0]['value'];
      }
      else {
        $spammastermessage = $formstate->getValue('nachricht');
      }
    }
    if (!empty($formstate->getValue('description')) && NULL !== $formstate->getValue('description')) {
      if (is_array($formstate->getValue('description'))) {
        $spammastermessage = $formstate->getValue('description')[0]['value'];
      }
      else {
        $spammastermessage = $formstate->getValue('description');
      }
    }
    if (!empty($formstate->getValue('comment_body')) && NULL !== $formstate->getValue('comment_body')) {
      if (is_array($formstate->getValue('comment_body'))) {
        $spammastermessage = $formstate->getValue('comment_body')[0]['value'];
      }
      else {
        $spammastermessage = $formstate->getValue('comment_body');
      }
    }
    if (!empty($formstate->getValue('comments')) && NULL !== $formstate->getValue('comments')) {
      if (is_array($formstate->getValue('comments'))) {
        $spammastermessage = $formstate->getValue('comments')[0]['value'];
      }
      else {
        $spammastermessage = $formstate->getValue('comments');
      }
    }
    if (!empty($formstate->getValue('field_message')) && NULL !== $formstate->getValue('field_message')) {
      if (is_array($formstate->getValue('field_message'))) {
        $spammastermessage = $formstate->getValue('field_message')[0]['value'];
      }
      else {
        $spammastermessage = $formstate->getValue('field_message');
      }
    }
    if (!empty($formstate->getValue('additional_information')) && NULL !== $formstate->getValue('additional_information')) {
      if (is_array($formstate->getValue('additional_information'))) {
        $spammastermessage = $formstate->getValue('additional_information')[0]['value'];
      }
      else {
        $spammastermessage = $formstate->getValue('additional_information');
      }
    }
    if (!empty($formstate->getValue('messaggio_')) && NULL !== $formstate->getValue('messaggio_')) {
      if (is_array($formstate->getValue('messaggio_'))) {
        $spammastermessage = $formstate->getValue('messaggio_')[0]['value'];
      }
      else {
        $spammastermessage = $formstate->getValue('messaggio_');
      }
    }
    if (!empty($formstate->getValue('messaggio')) && NULL !== $formstate->getValue('messaggio')) {
      if (is_array($formstate->getValue('messaggio'))) {
        $spammastermessage = $formstate->getValue('messaggio')[0]['value'];
      }
      else {
        $spammastermessage = $formstate->getValue('messaggio');
      }
    }
    if (!empty($formstate->getValue('toiawase')) && NULL !== $formstate->getValue('toiawase')) {
      if (is_array($formstate->getValue('toiawase'))) {
        $spammastermessage = $formstate->getValue('toiawase')[0]['value'];
      }
      else {
        $spammastermessage = $formstate->getValue('toiawase');
      }
    }
    if (!empty($formstate->getValue('project_brief')) && NULL !== $formstate->getValue('project_brief')) {
      if (is_array($formstate->getValue('project_brief'))) {
        $spammastermessage = $formstate->getValue('project_brief')[0]['value'];
      }
      else {
        $spammastermessage = $formstate->getValue('project_brief');
      }
    }
    if (!empty($formstate->getValue('notes')) && NULL !== $formstate->getValue('notes')) {
      if (is_array($formstate->getValue('notes'))) {
        $spammastermessage = $formstate->getValue('notes')[0]['value'];
      }
      else {
        $spammastermessage = $formstate->getValue('notes');
      }
    }
    if (!empty($formstate->getValue('additional_notes')) && NULL !== $formstate->getValue('additional_notes')) {
      if (is_array($formstate->getValue('additional_notes'))) {
        $spammastermessage = $formstate->getValue('additional_notes')[0]['value'];
      }
      else {
        $spammastermessage = $formstate->getValue('additional_notes');
      }
    }
    if (!empty($formstate->getValue('nature_of_enquiry')) && NULL !== $formstate->getValue('nature_of_enquiry')) {
      if (is_array($formstate->getValue('nature_of_enquiry'))) {
        $spammastermessage = $formstate->getValue('nature_of_enquiry')[0]['value'];
      }
      else {
        $spammastermessage = $formstate->getValue('nature_of_enquiry');
      }
    }
    if (!empty($formstate->getValue('description_of_problem')) && NULL !== $formstate->getValue('description_of_problem')) {
      if (is_array($formstate->getValue('description_of_problem'))) {
        $spammastermessage = $formstate->getValue('description_of_problem')[0]['value'];
      }
      else {
        $spammastermessage = $formstate->getValue('description_of_problem');
      }
    }
    if (!empty($formstate->getValue('questions_or_comments_')) && NULL !== $formstate->getValue('questions_or_comments_')) {
      if (is_array($formstate->getValue('questions_or_comments_'))) {
        $spammastermessage = $formstate->getValue('questions_or_comments_')[0]['value'];
      }
      else {
        $spammastermessage = $formstate->getValue('questions_or_comments_');
      }
    }
    if (!empty($formstate->getValue('messagio')) && NULL !== $formstate->getValue('messagio')) {
      if (is_array($formstate->getValue('messagio'))) {
        $spammastermessage = $formstate->getValue('messagio')[0]['value'];
      }
      else {
        $spammastermessage = $formstate->getValue('messagio');
      }
    }
    if (!empty($formstate->getValue('afmetingen')) && NULL !== $formstate->getValue('afmetingen')) {
      if (is_array($formstate->getValue('afmetingen'))) {
        $spammastermessage = $formstate->getValue('afmetingen')[0]['value'];
      }
      else {
        $spammastermessage = $formstate->getValue('afmetingen');
      }
    }
    if (!empty($formstate->getValue('position_applied_for')) && NULL !== $formstate->getValue('position_applied_for')) {
      if (is_array($formstate->getValue('position_applied_for'))) {
        $spammastermessage = $formstate->getValue('position_applied_for')[0]['value'];
      }
      else {
        $spammastermessage = $formstate->getValue('position_applied_for');
      }
    }
    if (!empty($formstate->getValue('mt_message')) && NULL !== $formstate->getValue('mt_message')) {
      if (is_array($formstate->getValue('mt_message'))) {
        $spammastermessage = $formstate->getValue('mt_message')[0]['value'];
      }
      else {
        $spammastermessage = $formstate->getValue('mt_message');
      }
    }
    if (!empty($formstate->getValue('additional_comments_')) && NULL !== $formstate->getValue('additional_comments_')) {
      if (is_array($formstate->getValue('additional_comments_'))) {
        $spammastermessage = $formstate->getValue('additional_comments_')[0]['value'];
      }
      else {
        $spammastermessage = $formstate->getValue('additional_comments_');
      }
    }
    if (!empty($formstate->getValue('how_do_you_intent_do_use_our_materials')) && NULL !== $formstate->getValue('how_do_you_intent_do_use_our_materials')) {
      if (is_array($formstate->getValue('how_do_you_intent_do_use_our_materials'))) {
        $spammastermessage = $formstate->getValue('how_do_you_intent_do_use_our_materials')[0]['value'];
      }
      else {
        $spammastermessage = $formstate->getValue('how_do_you_intent_do_use_our_materials');
      }
    }
    if (!empty($formstate->getValue('1_briefly_describe_the_proposed_change_add_a_separate_page_if_ne')) && NULL !== $formstate->getValue('1_briefly_describe_the_proposed_change_add_a_separate_page_if_ne')) {
      if (is_array($formstate->getValue('1_briefly_describe_the_proposed_change_add_a_separate_page_if_ne'))) {
        $spammastermessage = $formstate->getValue('1_briefly_describe_the_proposed_change_add_a_separate_page_if_ne')[0]['value'];
      }
      else {
        $spammastermessage = $formstate->getValue('1_briefly_describe_the_proposed_change_add_a_separate_page_if_ne');
      }
    }
    if (!empty($formstate->getValue('kform_nachricht')) && NULL !== $formstate->getValue('kform_nachricht')) {
      if (is_array($formstate->getValue('kform_nachricht'))) {
        $spammastermessage = $formstate->getValue('kform_nachricht')[0]['value'];
      }
      else {
        $spammastermessage = $formstate->getValue('kform_nachricht');
      }
    }
    if (!empty($formstate->getValue('text')) && NULL !== $formstate->getValue('text')) {
      if (is_array($formstate->getValue('text'))) {
        $spammastermessage = $formstate->getValue('text')[0]['value'];
      }
      else {
        $spammastermessage = $formstate->getValue('text');
      }
    }
    if (!empty($formstate->getValue('ihre_nachricht_kontakt')) && NULL !== $formstate->getValue('ihre_nachricht_kontakt')) {
      if (is_array($formstate->getValue('ihre_nachricht_kontakt'))) {
        $spammastermessage = $formstate->getValue('ihre_nachricht_kontakt')[0]['value'];
      }
      else {
        $spammastermessage = $formstate->getValue('ihre_nachricht_kontakt');
      }
    }
    if (!empty($formstate->getValue('field_ergaenzende_mitteilungen')) && NULL !== $formstate->getValue('field_ergaenzende_mitteilungen')) {
      if (is_array($formstate->getValue('field_ergaenzende_mitteilungen'))) {
        $spammastermessage = $formstate->getValue('field_ergaenzende_mitteilungen')[0]['value'];
      }
      else {
        $spammastermessage = $formstate->getValue('field_ergaenzende_mitteilungen');
      }
    }
    if (!empty($formstate->getValue('opmerkingen')) && NULL !== $formstate->getValue('opmerkingen')) {
      if (is_array($formstate->getValue('opmerkingen'))) {
        $spammastermessage = $formstate->getValue('opmerkingen')[0]['value'];
      }
      else {
        $spammastermessage = $formstate->getValue('opmerkingen');
      }
    }
    if (!empty($formstate->getValue('vraag')) && NULL !== $formstate->getValue('vraag')) {
      if (is_array($formstate->getValue('vraag'))) {
        $spammastermessage = $formstate->getValue('vraag')[0]['value'];
      }
      else {
        $spammastermessage = $formstate->getValue('vraag');
      }
    }
    if (!empty($formstate->getValue('question')) && NULL !== $formstate->getValue('question')) {
      if (is_array($formstate->getValue('question'))) {
        $spammastermessage = $formstate->getValue('question')[0]['value'];
      }
      else {
        $spammastermessage = $formstate->getValue('question');
      }
    }
    if (!empty($formstate->getValue('over_welke_persoon_p')) && NULL !== $formstate->getValue('over_welke_persoon_p')) {
      if (is_array($formstate->getValue('over_welke_persoon_p'))) {
        $spammastermessage = $formstate->getValue('over_welke_persoon_p')[0]['value'];
      }
      else {
        $spammastermessage = $formstate->getValue('over_welke_persoon_p');
      }
    }
    if (!empty($formstate->getValue('field_your_enquiry')) && NULL !== $formstate->getValue('field_your_enquiry')) {
      if (is_array($formstate->getValue('field_your_enquiry'))) {
        $spammastermessage = $formstate->getValue('field_your_enquiry')[0]['value'];
      }
      else {
        $spammastermessage = $formstate->getValue('field_your_enquiry');
      }
    }
    if (!empty($formstate->getValue('motivation_pour_participer_a_la_formation')) && NULL !== $formstate->getValue('motivation_pour_participer_a_la_formation')) {
      if (is_array($formstate->getValue('motivation_pour_participer_a_la_formation'))) {
        $spammastermessage = $formstate->getValue('motivation_pour_participer_a_la_formation')[0]['value'];
      }
      else {
        $spammastermessage = $formstate->getValue('motivation_pour_participer_a_la_formation');
      }
    }
    if (empty($spammastermessage)) {
      // If still no message try to replace with name.
      if (!empty($formstate->getValue('name')) && NULL !== $formstate->getValue('name')) {
        $spammastermessage = 'Content: ' . $formstate->getValue('name');
      }
      elseif (!empty($formstate->getValue('first_name')) && NULL !== $formstate->getValue('first_name')) {
        $spammastermessage = 'Content: ' . $formstate->getValue('first_name');
      }
      elseif (!empty($formstate->getValue('your_name')) && NULL !== $formstate->getValue('your_name')) {
        $spammastermessage = 'Content: ' . $formstate->getValue('your_name');
      }
      elseif (!empty($formstate->getValue('firstname')) && NULL !== $formstate->getValue('firstname')) {
        $spammastermessage = 'Content: ' . $formstate->getValue('firstname');
      }
      elseif (!empty($formstate->getValue('certification_of_accuracy')) && NULL !== $formstate->getValue('certification_of_accuracy')) {
        $spammastermessage = 'Content: ' . $formstate->getValue('certification_of_accuracy');
      }
      elseif (!empty($formstate->getValue('ihr_name')) && NULL !== $formstate->getValue('ihr_name')) {
        if (is_array($formstate->getValue('ihr_name'))) {
          $spammastermessage = $formstate->getValue('ihr_name')['first']['value'];
        }
        else {
          $spammastermessage = $formstate->getValue('ihr_name');
        }
      }
      elseif (!empty($formstate->getValue('personalien')) && NULL !== $formstate->getValue('personalien')) {
        if (is_array($formstate->getValue('personalien'))) {
          $spammastermessage = $formstate->getValue('personalien')['first'];
        }
        else {
          $spammastermessage = $formstate->getValue('personalien');
        }
      }
      elseif (!empty($formstate->getValue('reason_of_contact')) && NULL !== $formstate->getValue('reason_of_contact')) {
        if (is_array($formstate->getValue('reason_of_contact'))) {
          $spammastermessage = $formstate->getValue('reason_of_contact')[0]['value'];
        }
        else {
          $spammastermessage = $formstate->getValue('reason_of_contact');
        }
      }
      elseif (!empty($formstate->getValue('motivatie')) && NULL !== $formstate->getValue('motivatie')) {
        if (is_array($formstate->getValue('motivatie'))) {
          $spammastermessage = $formstate->getValue('motivatie')[0]['value'];
        }
        else {
          $spammastermessage = $formstate->getValue('motivatie');
        }
      }
      elseif (!empty($formstate->getValue('how_can_we_help_you_')) && NULL !== $formstate->getValue('how_can_we_help_you_')) {
        if (is_array($formstate->getValue('how_can_we_help_you_'))) {
          $spammastermessage = $formstate->getValue('how_can_we_help_you_')[0]['value'];
        }
        else {
          $spammastermessage = $formstate->getValue('how_can_we_help_you_');
        }
      }
      elseif (!empty($formstate->getValue('location_of_event')) && NULL !== $formstate->getValue('location_of_event')) {
        if (is_array($formstate->getValue('location_of_event'))) {
          $spammastermessage = $formstate->getValue('location_of_event')[0]['value'];
        }
        else {
          $spammastermessage = $formstate->getValue('location_of_event');
        }
      }
      elseif (!empty($formstate->getValue('civicrm_1_activity_1_activity_details')) && NULL !== $formstate->getValue('civicrm_1_activity_1_activity_details')) {
        if (is_array($formstate->getValue('civicrm_1_activity_1_activity_details'))) {
          $spammastermessage = $formstate->getValue('civicrm_1_activity_1_activity_details')['value'];
        }
        else {
          $spammastermessage = $formstate->getValue('civicrm_1_activity_1_activity_details');
        }
      }
      elseif (!empty($formstate->getValue('what_units_are_you_interested_in_and_when_')) && NULL !== $formstate->getValue('what_units_are_you_interested_in_and_when_')) {
        if (is_array($formstate->getValue('what_units_are_you_interested_in_and_when_'))) {
          $spammastermessage = $formstate->getValue('what_units_are_you_interested_in_and_when_')['value'];
        }
        else {
          $spammastermessage = $formstate->getValue('what_units_are_you_interested_in_and_when_');
        }
      }
      elseif (!empty($formstate->getValue('details_of_your_quote_request_')) && NULL !== $formstate->getValue('details_of_your_quote_request_')) {
        if (is_array($formstate->getValue('details_of_your_quote_request_'))) {
          $spammastermessage = $formstate->getValue('details_of_your_quote_request_')['value'];
        }
        else {
          $spammastermessage = $formstate->getValue('details_of_your_quote_request_');
        }
      }
      elseif (!empty($formstate->getValue('civicrm_1_contact_1_cg2_custom_124')) && NULL !== $formstate->getValue('civicrm_1_contact_1_cg2_custom_124')) {
        if (is_array($formstate->getValue('civicrm_1_contact_1_cg2_custom_124'))) {
          $spammastermessage = $formstate->getValue('civicrm_1_contact_1_cg2_custom_124')['value'];
        }
        else {
          $spammastermessage = $formstate->getValue('civicrm_1_contact_1_cg2_custom_124');
        }
      }
      elseif (!empty($formstate->getValue('name')) && NULL !== $formstate->getValue('name')) {
        if (is_array($formstate->getValue('name'))) {
          $spammastermessage = 'Content: ' . $formstate->getValue('name')['first'];
        }
        else {
          $spammastermessage = 'Content: ' . $formstate->getValue('name');
        }
      }
      elseif (!empty($formstate->getValue('prenom')) && NULL !== $formstate->getValue('prenom')) {
        $spammastermessage = 'Content: ' . $formstate->getValue('prenom');
      }
      elseif (!empty($formstate->getValue('mt_first_name')) && NULL !== $formstate->getValue('mt_first_name')) {
        $spammastermessage = 'Content: ' . $formstate->getValue('mt_first_name');
      }
      elseif (!empty($formstate->getValue('who_is_submitting_this_request_')) && NULL !== $formstate->getValue('who_is_submitting_this_request_')) {
        $spammastermessage = 'Content: ' . $formstate->getValue('who_is_submitting_this_request_');
      }
      elseif (!empty($formstate->getValue('naam')) && NULL !== $formstate->getValue('naam')) {
        $spammastermessage = 'Content: ' . $formstate->getValue('naam');
      }
      elseif (!empty($formstate->getValue('nome_e_cognome')) && NULL !== $formstate->getValue('nome_e_cognome')) {
        $spammastermessage = 'Content: ' . $formstate->getValue('nome_e_cognome');
      }
      elseif (!empty($formstate->getValue('name_first_last_')) && NULL !== $formstate->getValue('name_first_last_')) {
        $spammastermessage = 'Content: ' . $formstate->getValue('name_first_last_');
      }
      elseif (!empty($formstate->getValue('vorname_und_name')) && NULL !== $formstate->getValue('vorname_und_name')) {
        $spammastermessage = 'Content: ' . $formstate->getValue('vorname_und_name');
      }
      elseif (!empty($formstate->getValue('nom_i_cognoms')) && NULL !== $formstate->getValue('nom_i_cognoms')) {
        $spammastermessage = 'Content: ' . $formstate->getValue('nom_i_cognoms');
      }
      else {
        $spammastermessage = 'contentless';
      }
    }
    $prespamformdata = [
      'spammasteremail'   => $spammasteremail,
      'spammastermessage' => $spammastermessage,
    ];
    $spamformdata = Json::encode($prespamformdata);

    return $spamformdata;

  }

}
